<?php

/**
 * Menu callback. Returns JSON for an issue node.
 *
 * @todo
 * When the major API version of this callback changes, it will need to be
 * refactored. See http://drupal.org/node/112805#comment-5469522.
 *
 * @param $issue
 *   A node of type project_issue.
 * @return void
 */
function project_issue_json($issue) {
  if ($project = node_load($issue->project_issue['pid'])) {
    // All returned URLs need to be absolute.
    $options = array('absolute' => TRUE);

    // Retrieve the associated project release.
    if (isset($issue->project_issue['rid'])) {
      $release = node_load($issue->project_issue['rid']);
    }
    if (empty($release)) {
      $release = (object) array(
        'nid' => NULL,
        'project_release' => array('version' => NULL),
      );
    }

    // Retrieve the assigned user.
    if (isset($issue->project_issue['assigned'])) {
      $assigned = user_load($issue->project_issue['assigned']);
    }
    if (empty($assigned)) {
      $assigned = (object) array(
        'uid' => NULL,
        'name' => NULL,
      );
    }
    $assigned->url = !empty($assigned->uid) ? url('user/' . $assigned->uid, $options) : NULL;

    // Retrieve the issue author.
    if (!empty($issue->uid)) {
      $author = user_load($issue->uid);
    }
    if (empty($author)) {
      $author = (object) array(
        'uid' => NULL,
        'name' => NULL,
      );
    }
    $author->url = !empty($author->uid) ? url('user/' . $author->uid, $options) : NULL;

    // Retrieve file attachments on the issue node.
    // Does not contain attachments on comments.
    // @see comment_upload_project_issue_json_alter()
    $attachments = array();
    foreach ($issue->files as $fid => $file) {
      // Attachments are keyed by comment ID. We treat the original issue node
      // as post 0, so as to have all issue attachments in one property that can
      // be processed by API consumers.
      $attachments[0]['contributorId'] = $author->uid;
      $attachments[0]['commentId'] = 0;
      $attachments[0]['urls'][$fid] = file_create_url($file->filepath);
    }

    // Populate basic issue data.
    $return = array(
      // Increase minor version when adding information.
      // Increase major version when breaking backwards compatibility.
      'apiVersion' => '1.0',
      'title' => $issue->title,
      'id' => $issue->nid,
      'created' => $issue->created,
      'changed' => $issue->changed,
      'url' => url('node/' . $issue->nid, $options),
      'component' => $issue->project_issue['component'],
      'category' => $issue->project_issue['category'],
      'priorityId' => $issue->project_issue['priority'],
      'priority' => project_issue_priority($issue->project_issue['priority']),
      'statusId' => $issue->project_issue['sid'],
      'status' => project_issue_state($issue->project_issue['sid']),
      'versionId' => $release->nid,
      'version' => $release->project_release['version'],
      'authorId' => $author->uid,
      'authorName' => $author->name,
      'authorUrl' => $author->url,
      'assignedId' => $assigned->uid,
      'assignedName' => $assigned->name,
      'assignedUrl' => $assigned->url,
      'projectId' => $project->nid,
      'projectTitle' => $project->title,
      'projectUrl' => url('node/' . $project->nid, $options),
      'projectName' => $project->project['uri'],
      'comments' => array(),
      'contributors' => array(),
      'attachments' => $attachments,
    );

    // Retrieve and populate comments and contributors (comment authors).
    $result = db_query("SELECT c.cid, c.pid, c.subject, c.status, c.timestamp, c.thread, u.name, u.uid FROM {comments} c INNER JOIN {users} u ON c.uid = u.uid WHERE c.nid = %d AND c.status = %d ORDER BY c.cid ASC", $issue->nid, COMMENT_PUBLISHED);
    while ($row = db_fetch_object($result)) {
      $return['contributors'][$row->uid] = array(
        'id' => $row->uid,
        'name' => $row->name,
        'url' => url('user/' . $row->uid, $options),
      );
      $return['comments'][$row->cid] = array(
        'id' => $row->cid,
        'contributorId' => $row->uid,
        'url' => url('node/' . $issue->nid, array('absolute' => TRUE, 'fragment' => 'comment-' . $row->cid)),
        'created' => $row->timestamp,
        // D6: 0 is COMMENT_PUBLISHED.
        'status' => (int) !$row->status,
        'subject' => $row->subject,
      );
    }

    // Allow modules to change or enhance the returned data.
    $context = array(
      'issue' => $issue,
      'project' => $project,
      'release' => $release,
      'assigned' => $assigned,
      'author' => $author,
    );
    drupal_alter('project_issue_json', $return, $context);

    drupal_json($return);
  }
}
